@@ -1,10 +1,9 @@
 # @package      hubzero-submit-common
 # @file         MessageCore.py
-# @author       Steven Clark <clarks@purdue.edu>
-# @copyright    Copyright (c) 2012-2015 HUBzero Foundation, LLC.
+# @copyright    Copyright (c) 2012-2020 The Regents of the University of California.
 # @license      http://opensource.org/licenses/MIT MIT
 #
-# Copyright (c) 2012-2015 HUBzero Foundation, LLC.
+# Copyright (c) 2012-2020 The Regents of the University of California.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +23,7 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 #
-# HUBzero is a registered trademark of HUBzero Foundation, LLC.
+# HUBzero is a registered trademark of The Regents of the University of California.
 #
 from OpenSSL import SSL
 import sys
@@ -265,7 +264,16 @@ class MessageCore:
             listenerChannel.close()
          listenerChannel = None
       except Exception as e:
-         if e.args[0] == 111:
+         if   e.args[0] == 110:
+#           connection timed out
+            if logErrorMessage:
+               self.logger.log(logging.ERROR,getLogMessage("openListenerChannel: %s" % (e.args[1])))
+         elif e.args[0] == 111:
+#           connection refused
+            if logErrorMessage:
+               self.logger.log(logging.ERROR,getLogMessage("openListenerChannel: %s" % (e.args[1])))
+         elif e.args[0] == 13:
+#           permission denied
             if logErrorMessage:
                self.logger.log(logging.ERROR,getLogMessage("openListenerChannel: %s" % (e.args[1])))
          else:
@@ -306,7 +314,9 @@ class MessageCore:
                                    bufferSize):
       message = ""
 
-      while True:
+      nReceives = 0
+      maximumReceives = 8
+      while nReceives < maximumReceives:
          try:
             messageChunk = channel.recv(bufferSize)
          except socket.error as e:
@@ -339,6 +349,7 @@ class MessageCore:
                message = None
             break
          else:
+            nReceives += 1
             if messageChunk:
                message += messageChunk.decode('utf-8')
             else:
@@ -346,6 +357,8 @@ class MessageCore:
                   message = None
                break
 
+#     self.logger.log(logging.DEBUG,getLogMessage("%d buffers read(%d)" % (nReceives,bufferSize)))
+
       return(message)
 
 
@@ -370,11 +383,20 @@ class MessageCore:
                else:
                   message = None
                break
-      except:
-         self.logger.log(logging.ERROR,getLogMessage("Unexpected error in receiveBlockingMessage() " + message))
-         self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
-         if message == "":
+      except Exception as e:
+         if   e.args[0] == 104:
+#           connection reset by peer
+            self.logger.log(logging.ERROR,getLogMessage("__receiveBlockingMessage: %s" % (e.args[1])))
             message = None
+         elif e.args[0] == 4:
+#           interrupted system call
+            self.logger.log(logging.ERROR,getLogMessage("__receiveBlockingMessage: %s" % (e.args[1])))
+            message = None
+         else:
+            self.logger.log(logging.ERROR,getLogMessage("Unexpected error in receiveBlockingMessage() " + message))
+            self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
+            if message == "":
+               message = None
 
       return(message)
 
@@ -404,6 +426,7 @@ class MessageCore:
                                 channel,
                                 message):
       try:
+         message = message.encode('utf-8')
          transmittedLength = channel.send(message)
          if transmittedLength == 0:
             self.logger.log(logging.ERROR,getLogMessage("socket connection broken in sendNonBlockingMessage()"))
@@ -411,21 +434,23 @@ class MessageCore:
       except (SSL.WantReadError, SSL.WantWriteError, SSL.WantX509LookupError):
          transmittedLength = 0
       except SSL.Error:
-         self.logger.log(logging.ERROR,getLogMessage("Unexpected SSL error in sendNonBlockingMessage(%s)" % (message[0:20])))
+         self.logger.log(logging.ERROR,getLogMessage("Unexpected SSL error in sendNonBlockingMessage(%s)" % (message[0:50])))
          self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
          transmittedLength = 0
       except Exception as e:
          if isinstance(e.args,tuple):
             if not e.args[0] in [EPIPE]:
-               self.logger.log(logging.ERROR,getLogMessage("Unexpected error in sendNonBlockingMessage(%s)" % (message[0:20])))
+               self.logger.log(logging.ERROR,getLogMessage("Unexpected error in sendNonBlockingMessage(%s)" % (message[0:50])))
                self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
             else:
                self.logger.log(logging.ERROR,getLogMessage("sendNonBlockingMessage: Broken pipe"))
          else:
-            self.logger.log(logging.ERROR,getLogMessage("Unexpected error in sendNonBlockingMessage(%s)" % (message[0:20])))
+            self.logger.log(logging.ERROR,getLogMessage("Unexpected error in sendNonBlockingMessage(%s)" % (message[0:50])))
             self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
          transmittedLength = -1
 
+#     self.logger.log(logging.DEBUG,getLogMessage("buffers written(%d): %s" % (transmittedLength,message[0:100])))
+
       return(transmittedLength)
 
 
@@ -436,6 +461,7 @@ class MessageCore:
       try:
          if fixedBufferSize > 0:
             fixedBufferMessage = "%-*s" % (fixedBufferSize,message)
+            fixedBufferMessage = fixedBufferMessage.encode('utf-8')
             bytesRemaining = fixedBufferSize
             while bytesRemaining:
                transmittedLength = channel.send(fixedBufferMessage[fixedBufferSize-bytesRemaining:])
@@ -445,9 +471,10 @@ class MessageCore:
                   transmittedLength = -1
                   break
          else:
-            bytesRemaining = len(message)
+            transmissionMessage = message.encode('utf-8')
+            bytesRemaining = len(transmissionMessage)
             while bytesRemaining:
-               transmittedLength = channel.send(message[len(message)-bytesRemaining:])
+               transmittedLength = channel.send(transmissionMessage[len(transmissionMessage)-bytesRemaining:])
                bytesRemaining -= transmittedLength
                if transmittedLength == 0:
                   self.logger.log(logging.ERROR,getLogMessage("socket connection broken in sendBlockingMessage()"))
@@ -456,23 +483,23 @@ class MessageCore:
       except SSL.Error as e:
          if isinstance(e.args,tuple):
             if not e.args[0] in [EPIPE]:
-               self.logger.log(logging.ERROR,getLogMessage("Unexpected error in sendBlockingMessage(%s)" % (message[0:20])))
-               self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
+               self.logger.log(logging.ERROR,getLogMessage("Unexpected error in sendBlockingMessage(%s)" % (message[0:50])))
+               self.logger.log(logging.ERROR,getLogMessage("%s %s" % (e.args[0][0][1],e.args[0][0][2])))
             else:
                self.logger.log(logging.ERROR,getLogMessage("sendBlockingMessage: Broken pipe"))
          else:
-            self.logger.log(logging.ERROR,getLogMessage("Unexpected error in sendBlockingMessage(%s)" % (message[0:20])))
+            self.logger.log(logging.ERROR,getLogMessage("Unexpected error in sendBlockingMessage(%s)" % (message[0:50])))
             self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
          transmittedLength = -1
       except Exception as e:
          if isinstance(e.args,tuple):
             if not e.args[0] in [EPIPE]:
-               self.logger.log(logging.ERROR,getLogMessage("Unexpected error in sendBlockingMessage(%s)" % (message[0:20])))
+               self.logger.log(logging.ERROR,getLogMessage("Unexpected error in sendBlockingMessage(%s)" % (message[0:50])))
                self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
             else:
                self.logger.log(logging.ERROR,getLogMessage("sendBlockingMessage: Broken pipe"))
          else:
-            self.logger.log(logging.ERROR,getLogMessage("Unexpected error in sendBlockingMessage(%s)" % (message[0:20])))
+            self.logger.log(logging.ERROR,getLogMessage("Unexpected error in sendBlockingMessage(%s)" % (message[0:50])))
             self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
          transmittedLength = -1
 
@@ -556,14 +583,15 @@ class MessageCore:
          if nSent > 0:
             # Expect the same message back.
             reply = self.receiveMessage(channel,nSent,nSent)
-            if len(reply) == len(message):
-               if reply[0:7] == message[0:7]:
-                  valid = True
-                  if blocking:
-                     channel.setblocking(1)
-                  else:
-                     channel.setblocking(0)
-                  self.defaultBufferSize = int(reply.split()[-1])
+            if reply:
+               if len(reply) == len(message):
+                  if reply[0:7] == message[0:7]:
+                     valid = True
+                     if blocking:
+                        channel.setblocking(1)
+                     else:
+                        channel.setblocking(0)
+                     self.defaultBufferSize = int(reply.split()[-1])
       except Exception:
          self.logger.log(logging.ERROR,getLogMessage("Connection initiateHandshake failed.  Protocol mismatch?"))
          self.logger.log(logging.ERROR,getLogMessage("initiateHandshake(%s): %s" % (message.strip(),str(reply).strip())))
